import { LitElement, css, html } from 'lit';
import { customElement } from 'lit/decorators.js';

const mosaic_WIDGETS = 'mosaic-widgets';

@customElement('mosaic-configure')
export default class mosaicConfigure extends LitElement {
  text = '[]'

  constructor() {
    super();
    let that = this;

    setTimeout( async () => {
      const response = await fetch('CONFIG_FETCH_URL');
      that.text = await response.text();
      that.requestUpdate();
    }, 100);
  }

  static override styles = [
    css`
       .centerit {
        text-align: center;
      }

      @media (max-width: 601px)  {
        .widgets {
          display: display !important;
        }

        .widgets >* {
          margin: .6em !important;
        }
      }
      @media (max-width: 501px) and (display-mode: standalone) {
        .widgets {
          display: block !important;
        }
      }

      .buttonbar {
        display: flex;
        width: 100%;
        justify-content: center;
        flex-wrap: wrap;
        flex-direction: row;
      }

      .widgets {
        display: flex;
        flex-wrap: wrap;
        flex-direction: row;
      }

      .widgets >* {
        flex: 1 1 300px;
        margin: .2em;
        padding: .5em;
        border: .1em solid black;
        border-radius: .5em;
      }

      .widget-edit {
        display: flex;
        flex-direction: row;
      }

      .widget-edit >* {
        margin: .2em;
        padding: .5em;
      }

      input {
        width: 100%;
        padding: .5em;
        border-radius: .5em;
      }

      .button {
        padding: .5em;
        padding-left: 1.2em;
        padding-right: 1.2em;
        border-radius: .5em;
      }
    `
  ];

  handleFiles(event: any) {
    if (event?.target?.files?.length > 0) {
      const file = event?.target?.files[0];
      const reader = new FileReader();
      reader.onload = () => {
        localStorage.setItem(mosaic_WIDGETS, reader?.result as string ?? '[]');
        window.location.replace('./index.html');
      }
      reader.readAsText(file);
    }
  }

  save() {
    const elems = this.shadowRoot?.children ?? null;
    const widgets = elems ? elems?.item(2)?.children : null;
    const widgetsLen = widgets ? widgets?.length : 0
    const mosaicWidgets = [];

    for (let widgetI = 0; widgetI < widgetsLen; widgetI++) {
      const elem = widgets?.item(widgetI);
      const titleEditElem = elem?.children.item(0)?.children.item(1) as HTMLInputElement;
      const urlEditElem = elem?.children.item(1)?.children.item(1) as HTMLInputElement;
      const heightEditElem = elem?.children.item(2)?.children.item(1) as HTMLInputElement;
      const title = titleEditElem.value;
      const url = urlEditElem.value;
      const height = heightEditElem.value;
      if (title && title.trim() !== '' && url && url.trim() !== '') {
        mosaicWidgets.push({
          title,
          url,
          height,
        });
      }
    }

    fetch(`CONFIG_STORE_URL?json=${JSON.stringify(mosaicWidgets)}`).then(() => window.location.replace('./index.html'));
  }

  override render() {
    const widgetJSON = this.text ?? '[]';
    const widgets = JSON.parse(widgetJSON);

    return html`
    <h1 class="centerit">mosaic Widget Editor</h1>
    <div class="buttonbar"><button class="button" @click="${async () => await this.save()}">Save</button></div>
    <div class="widgets">
      ${widgets.map((widget: any) => {
      return html`
          <div name="widget-edit">
            <div><div>Title</div><input type="text" name="titleEdit" value="${widget.title}"></div>
            <div><div>Href</div><input type="text" name="urlEdit" value="${widget.url}"></div>
            <div><div>Height</div><input type="text" name="urlHeight" value="${widget.height}"></div>
          </div>
      `;
    })}
      <div name="widget-edit">
        <div><div>Title </div><input type="text" name="titleEdit"></div >
        <div><div>Href </div><input type="text" name="urlEdit"></div >
        <div><div>Height</div><input type="text" name="urlHeight"></div>
      </div>
    </div>
    <div class="buttonbar"><button class="button" @click="${() => this.save()}">Save</button></div>
    <a id="downloader" style="display: none"></a>
    <input type="file" @change="${(e: any) => this.handleFiles(e)}" id="importer" style="display:none" />
    `;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'mosaic-configure': mosaicConfigure;
  }
}