import { LitElement, css, html } from 'lit';
import { customElement } from 'lit/decorators.js';

@customElement('mosaic-widgets')
export class mosaicWidgets extends LitElement {
  widgets = []

  constructor() {
    super();

    let that = this;

    setTimeout(async () => {
      const response = await fetch('CONFIG_FETCH_URL');
      const text = await response.text();
  
      const widgetJSON = text ?? '[]';
      that.widgets = JSON.parse(widgetJSON);
      if (that.widgets.length == 0) {
        window.location.href = './edit.html';
      }

      that.requestUpdate();
    }, 100);
  }

  static override styles = [
    css`
      .centerit {
        text-align: center;
      }

      @media (max-width: 601px)  {
        .widgets {
          display: display !important;
        }

        .widgets >* {
          margin: .6em !important;
        }
      }
      @media (max-width: 501px) and (display-mode: standalone) {
        .widgets {
          display: block !important;
        }
      }

      .header {
        background-color: #146b37;
        color: white;
        text-align: center;
        border: .01em solid black;
        padding: .5em;
      }

      .widgets {
        display: flex;
        flex-wrap: wrap;
        flex-direction: row;
        justify-content: space-between;
        align-content: space-between;
      }

      .widget-col {
        flex: 1;
      }

      .widgets >* {
        align-self: center;
        flex: 1 1 160px;
        margin: .2em;
      }
    `
  ];

  override render() {
    return html`
    <h1 class="centerit">Mosaic</h1>
    <div class="widgets">
    ${this.widgets.map((widget: any) => {
      return html`<iframe width="100%" framespacing="0" frameborder="no" style="${widget.height ? `height: ${widget.height}` : ''}" src="${widget.url}" title="${widget.title}">${widget.title}</iframe>`;
    })}
    </div>
    `;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'mosaic-widgets': mosaicWidgets;
  }
}